/**
  * COPYRIGHT 2010 Nuvoton Technology Corp.
  ******************************************************************************
  * @file    DrvTK.h
  * @author  NuMicro MCU Software Team
  * @version V1.0.0
  * @date    08/11/2010
  * @brief   NANO1XX touch key driver header file
  ******************************************************************************
  */ 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "nano1xx.h"

#ifndef _NANO1XX_TK_H_
#define _NANO1XX_TK_H_


#ifdef  __cplusplus
extern "C"
{
#endif


#define E_DRVTK_ALREADY       _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTK, 1)  /** Already opened/closed     */
#define E_DRVTK_PARAM         _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTK, 2)  /** Invalid parameter         */
#define E_DRVTK_PERM          _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTK, 3)  /** No permision              */
#define E_DRVTK_BUSY          _SYSINFRA_ERRCODE(TRUE, MODULE_ID_DRVTK, 4)  /** TK engine busy            */

/** Max valid channel number. NUC200 supports 16 touch key channels, 0~15 */
#define TK_MAX_CH      15

/* version definition with SysInfra */
#define	DRVTK_MAJOR_NUM 1
#define	DRVTK_MINOR_NUM 00
#define	DRVTK_BUILD_NUM 000
#define DRVTK_VERSION_NUM    _SYSINFRA_VERSION(DRVTK_MAJOR_NUM, DRVTK_MINOR_NUM, DRVTK_BUILD_NUM)

/**
 * Indicates the touch key counter width. The wider the better sensitivity, but slower response time
 */
typedef enum {
    TK_TIMER_10BIT = 0,    ///< 10 bits
    TK_TIMER_12BIT,        ///< 12 bits
    TK_TIMER_14BIT,        ///< 14 bits
    TK_TIMER_16BIT         ///< 16 bits
}E_TK_COUNTER_WIDTH;

/**
 * Indicates the current level for conversion. The higher the better sensitivity
 */
typedef enum {
    TK_CURRENT_1_OVER_15 = 1,   ///< 1/15 charge/discharge current
    TK_CURRENT_2_OVER_15,       ///< 2/15 charge/discharge current 
    TK_CURRENT_3_OVER_15,       ///< 3/15 charge/discharge current
    TK_CURRENT_4_OVER_15,       ///< 4/15 charge/discharge current
    TK_CURRENT_5_OVER_15,       ///< 5/15 charge/discharge current
    TK_CURRENT_6_OVER_15,       ///< 6/15 charge/discharge current
    TK_CURRENT_7_OVER_15,       ///< 7/15 charge/discharge current
    TK_CURRENT_8_OVER_15,       ///< 8/15 charge/discharge current
    TK_CURRENT_9_OVER_15,       ///< 9/15 charge/discharge current
    TK_CURRENT_10_OVER_15,      ///< 10/15 charge/discharge current
    TK_CURRENT_11_OVER_15,      ///< 11/15 charge/discharge current
    TK_CURRENT_12_OVER_15,      ///< 12/15 charge/discharge current
    TK_CURRENT_13_OVER_15,      ///< 13/15 charge/discharge current
    TK_CURRENT_14_OVER_15,      ///< 14/15 charge/discharge current
    TK_CURRENT_FULL             ///< Full scale charge/discharge current
}E_TK_CURRENT_LEVEL;

/** Indicates touch key engine clock source */
typedef enum {
    TK_EXT_12M = 0,  ///< TK clock source from external 12MHz crystal clock
/*  TK_EXT_32K = 1,  ///< TK clock source from external 32kHz crystal clock. Do not use this clock source for TK */
    TK_PLL = 2,      ///< TK clock source from PLL clock
    TK_INT_12M = 3   ///< TK clock source from internal 12MHz crystal clock
}E_TK_CLK_SRC;

/** Indicates frequency divider */
typedef enum {
    TK_FREQ_DIV_1 = 0,
    TK_FREQ_DIV_2 = 1,
    TK_FREQ_DIV_4 = 2,
    TK_FREQ_DIV_8 = 3
}E_TK_TIME_FREQ_SEL;


/**
 * Stores TK clock settings
 */
typedef struct {
    E_TK_CLK_SRC eSrc;                 ///< TK engine clock source
    uint8_t u8Div;                      ///< TK engine clock divider
    E_TK_TIME_FREQ_SEL eFreqSel;      ///< Touch Key Time-Base Clock Frequency select
}S_TK_CLK_CFG;


/**
 * Stores touch key configurations related to sensitivity level
 */
typedef struct {
    E_TK_COUNTER_WIDTH eWidth;
    E_TK_CURRENT_LEVEL eLevel;  
    S_TK_CLK_CFG sClk;
    uint8_t u8Cap;
}S_TK_SENSE_CFG;

/** Touch key interrupt callback function prototype */
typedef void (*PFN_TK_CALLBACK)(uint32_t u32Param);

extern int32_t DrvTK_Open(void);
extern int32_t DrvTK_Close(void);
extern void DrvTK_Reset(void);
extern int32_t DrvTK_Start(uint8_t u8Ch, S_TK_SENSE_CFG *psCfg);
extern int32_t DrvTK_SetCounterWidth(E_TK_COUNTER_WIDTH eWidth);
extern int32_t DrvTK_SetCurrentLevel(E_TK_CURRENT_LEVEL eLevel);
extern int32_t DrvTK_SetClockFreq(S_TK_CLK_CFG *psClk);
extern int32_t DrvTK_SetExtCap(uint8_t u8Cap);
extern int32_t DrvTK_RegisterCallback(PFN_TK_CALLBACK pfnFp, uint32_t u32Param);
extern int32_t DrvTK_EnableInterrupt(void);
extern int32_t DrvTK_DisableInterrupt(void);
extern int32_t DrvTK_ReadData(uint16_t *pu16Data);
extern int32_t DrvTK_ReadStatus(uint8_t *pu8Level, uint8_t *pu8State);
#ifdef  __cplusplus
}
#endif

#endif //#ifndef _NANO1XX_TK_H_

/*** (C) COPYRIGHT 2010 Nuvoton Technology Corp. ***/
